unit hotparse;

{text parsing unit}
{$I HOTIMG.INC}
interface
uses classes, hotimage;

procedure ReadHSfromFile(const FileName: string; HotSpots: THotSpots);

procedure WriteHStoFile(const FileName:string; HotSpots: THotSpots);

implementation

uses sysutils,windows;
{$H+}

type
  TCharSet=set of Char;
  TValueKind=(vkID, vkHint, vkSpotKind, vkDisabled,
              vkPoints, vkMaskMouseOver, vkMaskMouseDown);

const

  BEGIN_TOKEN = '<';
  END_TOKEN = '>';
  MAP_SIGN = 'HOTMAP (HotImage v2.2)';
  VALUE_DELIM = ',';
  VALUE_ASSIGN = '=';
  VALUE_SPACE = ' ';
  STR_ARRAY_QUOTA = '"';


  WHITE_SPACES: TCharSet = [#9,#10,#13,VALUE_SPACE];

  ValuesToken: array [TValueKind] of string[9] = (
    'ID',
    'HINT',
    'SHAPE',
    'DISABLED',
    'COORDS',
    'MOUSEOVER',
    'MOUSEDOWN'
  );


  SPOT_KINDS: array[TSpotKind] of string[9]=('polyline','rectangle','ellipse');
  SPOT_MASKS: array[TMaskKind] of TValueKind=(vkMaskMouseOver,vkMaskMouseDown);

function PointToString(const P: TPoint):string;
begin
  Result:=format('%d,%d',[P.x,P.y]);
end;

{String routine from rx-lib}
function QuotedString(const S: string; Quote: Char): string;
{$IFDEF _D3_}
begin
  Result := AnsiQuotedStr(S, Quote);
{$ELSE}
var
  I: Integer;
begin
  Result := S;
  for I := Length(Result) downto 1 do
    if Result[I] = Quote then Insert(Quote, Result, I);
  Result := Quote + Result + Quote;
{$ENDIF}
end;




procedure WriteHStoFile(const FileName: string; HotSpots: THotSpots);
var
  f: TextFile;

  procedure WriteSpot(Spot: TSpot);

    procedure NextValue;
    begin
      writeln(f, VALUE_DELIM);
    end;

    procedure WritePointsParam(const ParamName: string; const P: PPoints; 
                               Cnt: Integer);
    var
      I: Integer;
    begin

      if Cnt <= 0 then exit;
      NextValue;
      write(f, ParamName, VALUE_ASSIGN, STR_ARRAY_QUOTA);
      for I := 0 to Cnt-1 do
      begin
        write(f,PointToString(P^[I]));
        if (I< Cnt-1) then
          if ((I+1) mod 8 = 0) then writeln(f)
            else write(f, VALUE_SPACE);
      end;
      write(f, STR_ARRAY_QUOTA);
    end;


    var
       mk:TMaskKind;
       mi:TMaskItem;
       tmpPt:TPoint;

    begin
      writeln(f);
      write(f, BEGIN_TOKEN);
      write(f, ValuesToken[vkID], VALUE_ASSIGN, Spot.SpotID);
      NextValue;
      write(f, ValuesToken[vkSpotKind], VALUE_ASSIGN, SPOT_KINDS[Spot.SpotKind]);
      WritePointsParam(ValuesToken[vkPoints], Spot.GetPointArray, Spot.Count);
      if not Spot.Enabled then
      begin
        NextValue;
        write(f, ValuesToken[vkDisabled]);
      end;

      if Spot.HintText <> '' then
      begin
        NextValue;
        write(f,ValuesToken[vkHint], VALUE_ASSIGN,
                QuotedString(Spot.HintText, STR_ARRAY_QUOTA));
      end;

      mi := Spot.MaskItem;
      for mk := low(TMaskKind) to high(TMaskKind) do
          if mk in mi.MaskKinds then
          begin
             tmpPt := mi.Offsets[mk];
             WritePointsParam(ValuesToken[SPOT_MASKS[mk]], PPoints(@tmpPt), 1);
          end;

      writeln(f, END_TOKEN);
    end;



var
  I, N: Integer;
begin
  AssignFile(f,FileName);
  Rewrite(f);
  try
    writeln(f,MAP_SIGN);
    N := HotSpots.Count - 1;
    for I:=0 to N do
     WriteSpot(HotSpots.Spots[I]);
  finally
    CloseFile(f);
  end;
end;



function SkipChars(const S: string;
                   var Index: Integer;
                   const CharSet: TCharSet): Boolean;
var
  I:integer;
begin
  Result:=false;
  for I:=Index to Length(S)do
  begin
    if not (S[I] in CharSet) then
    begin
      Index:=I;
      result:=true;
      break;
    end;
  end;
end;

function FindChars(const S: string;
                   var Index: Integer;
                   const CharSet: TCharSet): Boolean;
var
  I: integer;
begin
  result := false;
  for i := Index to Length(S) do
  begin
    if (S[i] in CharSet) then
    begin
      Index := I;
      Result := true;
      break;
    end;
  end;
  if not Result
    then Index := Length(S)+1;
end;

function GetValueKindFromString(const S: String;
                                var ValueKind: TValueKind): Boolean;
var
  vk: TValueKind;
begin
  Result := false;
  if length(s) = 0 then exit;
  for vk := low(vk) to high(vk) do
    if CompareText(ValuesToken[vk], s) = 0 then
    begin
      Result := true;
      ValueKind:=vk;
      break;
    end;
end;

function SpotKindFromString(const S: String): TSpotKind;
var
  sk: TSpotKind;
begin
  Result := skPolyline;
  for sk := low(TSpotKind) to High(TSpotKind) do
    if CompareText(SPOT_KINDS[sk], S) = 0 then
    begin
      Result := sk;
      break;
    end;
end;

function GetPointsCount(const S: string): Integer;
var
  I: integer;
begin
  I := 1;
  Result := 0;
  while FindChars(S, I, [VALUE_DELIM]) do
  begin
    inc(I);
    inc(result);
  end;
end;



procedure StringToPoints(const S: string; Pts: PPoint; Count: integer);
const
  Digits:TCharSet=['0'..'9', '-'];

  function LeftValue(Index: Integer): Integer;
  var
    I: Integer;
  begin
    I := Index - 1;
    while ( I >= 1) and (S[I] in Digits) do dec(I);
    inc(I);
    Result := StrToIntDef(Copy(S, I, Index-I), 0);
  end;

  function RightValue(Index:integer):integer;
  var
    I:integer;
  begin
    inc(Index);
    I:=Index;
    while(I<=Length(S)) and (S[I] in Digits) do inc(I);
    Result:=StrToIntDef(Copy(S,Index,I-Index),0);
  end;

var
  I: integer;
  PointIndex: integer;
  P: PPoint;
begin
  I := 1;
  PointIndex := 1;
  P := Pts;
  while (PointIndex <= Count) and FindChars(S, I, [VALUE_DELIM]) do
  begin
    P^.X := LeftValue(I);
    P^.Y := RightValue(I);
    inc(P);
    inc(PointIndex);
    inc(I);
  end;
end;




procedure ReadHSfromFile(const FileName: string; HotSpots: THotSpots);

var
 f: TextFile;
 Buffer: string;

 function ReadSpot: TSpot;

    function SkipWS(const AddtoSkip: TCharSet): Boolean;
    var
      I:Integer;
    begin
      I := 1;
      Result := SkipChars(Buffer, I, WHITE_SPACES + AddToSkip);
      while (not EOF(f)) and( not Result) do
      begin
        readln(f, Buffer);
        Result := SkipChars(Buffer, I, WHITE_SPACES + AddToSkip);
      end;


      if Result then
      begin
       if Buffer[I] = END_TOKEN then
       begin
         Result := false;
         inc(I);
       end;
       Delete(Buffer,1,I-1);
      end;
    end;


    procedure ExtractWord(var S: string);
    var
      I:integer;
    begin
      S := '';
      I := 1;
      FindChars(Buffer, i, WHITE_SPACES + [VALUE_ASSIGN, VALUE_DELIM, END_TOKEN]);
      S:=Trim(Copy(Buffer, 1, I-1));
      Delete(Buffer,1,I-1);
    end;

    function ReadValueString(var S: string): Boolean;

      procedure ReadQuoted;
      var
        tmpStr: string;
      begin
        if Buffer[1] = STR_ARRAY_QUOTA then
        begin
          S := '';
          Delete(Buffer, 1, 1);

          while true do
          begin
            if Length(Buffer) >= 1 then
            begin
              if  (Buffer[1] = STR_ARRAY_QUOTA) then
              begin
                Delete(Buffer, 1, 1);
                if (Length(Buffer) >= 1) and (Buffer[1] = STR_ARRAY_QUOTA) then
                begin
                  S := S + STR_ARRAY_QUOTA;
                  Delete(Buffer, 1, 1);
                end
                else break;
              end
               else
               begin
                 S := S + Buffer[1];
                 Delete(Buffer, 1, 1);
               end;
            end;
            if Length(Buffer) = 0 then
              if not EOF(f)then
              begin
                readln(f, tmpStr);
                Buffer := Buffer + #13#10 + tmpStr;
              end
               else break;

          end; //while true
        end
        else
          ExtractWord(S);
      end;


    begin

      result:=SkipWS([]);
      if result and (Buffer[1]=VALUE_ASSIGN) then
      begin
        Delete(Buffer,1,1);
        result:=SkipWS([]);
        if result then ReadQuoted;
      end;
    end;

    procedure ReadParams(Spot: TSpot);
    var
      ValueKind: TValueKind;
      Cnt, I: Integer;
      TokenFound: Boolean;
      S: string;
      PtBuffer, tmpPt: PPoint;
      MaskItem: TMaskItem;
    begin

      while SkipWS([VALUE_ASSIGN, VALUE_DELIM])do
      begin
        ExtractWord(S);
        TokenFound := GetValueKindFromString(S, ValueKind);
        {Some value found}
        if not TokenFound then Continue;
        if ValueKind = vkDisabled then
        begin
          Spot.Enabled := false;
          Continue;
        end;

        if not ReadValueString(S) then break;
        case ValueKind of
             vkID:
                Spot.SpotID := StrToIntDef(S,0);
             vkHint:
                Spot.HintText := S;
             vkSpotKind:
                Spot.SpotKind := SpotKindFromString(S);
             vkPoints:

                 begin
                   if Spot.SpotKind = skPolyLine then 
                       Cnt := GetPointsCount(s)
                    else
                       Cnt:=2;
                   if Cnt>0 then
                   begin
                     PtBuffer := AllocMem(Cnt * sizeof(TPoint));
                     try
                       StringToPoints(S, PtBuffer, Cnt);
                       if Spot.SpotKind = skPolyLine then
                       begin
                         Spot.Clear;
                         tmpPt:=PtBuffer;
                         for I := 1 to Cnt do
                         begin
                           Spot.AddPoint(tmpPt^);
                           inc(TmpPt);
                         end;
                       end
                        else
                        begin
                          With PRect(PtBuffer)^ do
                          begin
                            if Right < Left then
                            begin
                              I := Left;
                              Left := Right;
                              Right := I;
                            end; 
                            if Bottom< Top then
                            begin
                              I := Top;
                              Top := Bottom;
                              Bottom := I;
                            end; 
                          end;
                          Spot.BoundsRect := PRect(PtBuffer)^;
                        end;
                     finally
                       FreeMem(PtBuffer, Cnt * sizeof(TPoint));
                     end; 
                   end;
                 end;//vkPoints:

             vkMaskMouseOver..vkMaskMouseDown:
                begin
                  MaskItem:= Spot.MaskItem;
                  if ValueKind = vkMaskMouseOver then 
                  begin
                    PtBuffer := @MaskItem.Offsets[mkMouseOver];
                    Include(MaskItem.MaskKinds, mkMouseOver);
                  end
                   else
                   begin
                     PtBuffer:=@MaskItem.Offsets[mkMouseDown];
                     Include(MaskItem.MaskKinds, mkMouseDown);
                   end;
                  StringToPoints(S, PtBuffer, 1);
                  Spot.MaskItem:=MaskItem;
                end;
           end;//case
      end;//while
    end;


 var
   I: integer;
   S: string;
 begin
   Result:=nil;
   I := Pos(BEGIN_TOKEN, Buffer);
   if I > 0 then
   begin
      Result := TSpot.Create(skPolyLine);
      Delete(Buffer, 1, I);
   end
    else
    begin
      repeat
        readln(f, S);
        I:=Pos(BEGIN_TOKEN, S);
      until (I>0) or EOF(f);
      if I>0 then
      begin
        Result := TSpot.Create(skPolyLine);
        Buffer := Copy(S,I+1,length(S));
      end;
    end;
    if assigned(Result) then
    ReadParams(Result);
 end;


var
  Spot: TSpot;
begin
  Buffer := '';

  AssignFile(f, FileName);
  Reset(f);
  try
   Readln(F, Buffer);
   HotSpots.Clear;
   While true do
   begin
    Spot := ReadSpot;
    if assigned(Spot) then
      HotSpots.AddSpot(Spot)
    else break;
   end;


  finally
    CloseFile(f);
  end;
end;

end.