{*******************************************************}
{  HotImage Component Source                            }
{  Copyright (c) 1997,98 Miachin Alexander A.           }
{*******************************************************}

unit HSEdit;
{$I HOTIMG.INC}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls,HotImage;

type
  TfrmHSEdit = class(TForm)
    GroupBox1: TGroupBox;
    ScrollBox1: TScrollBox;
    btnOk: TButton;
    btnCancel: TButton;
    GroupBox2: TGroupBox;
    lboxSpots: TListBox;
    btnEditSpot: TButton;
    btnNewSpot: TButton;
    btnDeleteSpot: TButton;
    PaintBox1: TPaintBox;
    btnSave: TButton;
    btnLoad: TButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    btnClear: TButton;
    btnMask: TButton;
    procedure PaintBox1Paint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure lboxSpotsClick(Sender: TObject);
    procedure btnDeleteSpotClick(Sender: TObject);
    procedure btnEditSpotClick(Sender: TObject);
    procedure btnNewSpotClick(Sender: TObject);
    procedure lboxSpotsDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure lboxSpotsDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lboxSpotsDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure btnSaveClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnMaskClick(Sender: TObject);
  private
    { Private declarations }
    FGraphic,
    FMaskedGraphic: TGraphic;
    FPrivStr: string;
    FSize: TPoint;
    FNominator: Integer;
    FDenominator: Integer;
    FHotSpots: THotSpots;

    FSelectedSpot: TSpot;
    FDrawingSpot: TSpot;
    procedure PaintSpot;
    procedure DrawSpot(ASpot:TSpot);
    procedure FillLB;
    procedure UpdateButtons;
  public
    { Public declarations }
    function Edit(HotSpots:THotSpots;const S:String):boolean;

  end;


{ShowDialog function}
function EditHotSpotsDlg(HotSpots: THotSpots; const S: string): Boolean;

implementation
 uses  SpotEdit,imskedit, hotparse;

{$R *.DFM}

function EditHotSpotsDlg(HotSpots: THotSpots; const S: string): Boolean;
begin
 With  TfrmHSEdit.Create(Application)do
 try
   Result := Edit(HotSpots, S);
 finally
   Free;
 end;
end;



const LB_STR_LEN = 24;


function GetFirstWords(const S: String):string;
var 
  I: integer;
  Ch: Char;
begin
  result := '';
  I := 1;
  While (I <= Length(S)) and (I <= LB_STR_LEN) do
  begin
   Ch := S[I];
   If Ch > #13 then
    Result := Result + Ch
   else Result := Result+' ';
   inc(I);
  end;
  Result:=Result+'...';
end;

procedure TfrmHSEdit.UpdateButtons;
var 
  Flag: Boolean;
begin

  Flag:=
{$IFDEF FIX_LISTBOX_BUG}
   SendMessage(lBoxSpots.Handle, LB_GETCURSEL, 0, 0) >= 0;
{$ELSE}
   lBoxSpots.ItemIndex >=0;
{$ENDIF}


  btnEditSpot.Enabled := Flag;
  btnMask.Enabled := Flag;
  btnDeleteSpot.Enabled := flag;
end;


procedure TfrmHSEdit.FillLB;
var 
  i: Integer;
  Spot: TSpot;
begin
 With lboxSpots.Items do
 begin
   BeginUpdate;
   try
     Clear;
     For i := 0 to FHotSpots.Count - 1 do
     begin
       Spot := FHotSpots[i];
       AddObject(GetFirstWords(Spot.HintText), Spot);
     end;
  finally
     EndUpdate;
  end;
 end;

end;


procedure TfrmHSEdit.PaintSpot;
begin
  if FSelectedSpot <> nil then
  begin
    With PaintBox1.Canvas do
    begin
      Pen.Width := 2;
      Pen.Color := clBlack;
      Pen.Style := psSolid;
      Pen.Mode := pmNotXor;
      Brush.Style := bsClear;
    end;
    FDrawingSpot.DrawShape(PaintBox1.Canvas, Point(0,0), false);
  end;
end;

procedure TfrmHSEdit.DrawSpot(ASpot: TSpot);
var
   ScaleInfo: TScaleInfo;
begin
  if FSelectedSpot <> ASpot then
  begin
    //Painting old shape
    PaintSpot;
    FSelectedSpot := ASpot;

    If (ASpot <> nil) then
    begin
      With ScaleInfo do
      begin
        SrcSize.cx := FDenominator;
        SrcSize.cy := FDenominator;
        SrcOffset := Point(0,0);
        DstSize.cx := FNominator;
        DstSize.cy := FNominator;
        DstOffset := SrcOffset;
      end;
      FDrawingSpot.CopyShapeFrom(FSelectedSpot, ScaleInfo);
      //Painting New Shape
      PaintSpot;
    end;
  end;
end;



function TfrmHSEdit.Edit(HotSpots: THotSpots; const S: String): Boolean;

 procedure ScaleSize;
 begin
  With FGraphic do
  begin
    IF Width >= Height then
    begin
      FNominator := PaintBox1.Width;
      FDenominator := FGraphic.Width;
      FSize.X := FNominator;
      FSize.Y := MulDiv(FGraphic.Height, FNominator, FDenominator);
    end
     else
     begin
       FNominator := PaintBox1.Height;
       FDenominator := FGraphic.Height;
       FSize.X := MulDiv(FGraphic.Width, FNominator, FDenominator);
       FSize.Y := FNominator;
     end;
  end;
 end;

begin
   FPrivStr := S;
   Caption := S;
   FHotSpots.Assign(HotSpots);
   if (not assigned(HotSpots.Owner))or 
    (not assigned(HotSpots.Owner.Picture.Graphic)) then
   begin
     ShowMessage('Picture property undefined');
     Result := false;
     exit;
   end;
   FillLB;
   FGraphic := HotSpots.Owner.Picture.Graphic;
   FMaskedGraphic := HotSpots.Owner.MaskPicture.Graphic;
   ScaleSize;
   btnSave.Enabled := FHotSpots.Count > 0;
   btnClear.Enabled := FHotSpots.Count > 0;
   Result := ShowModal=mrOk;
   if Result then
     HotSpots.Assign(FHotSpots);
end;

procedure TfrmHSEdit.PaintBox1Paint(Sender: TObject);
begin
  PaintBox1.Canvas.StretchDraw(Rect(0,0,FSize.X,FSize.Y),FGraphic);
  PaintSpot;
end;

procedure TfrmHSEdit.FormCreate(Sender: TObject);
begin
  FHotSpots:=THotSpots.Create(nil);
  FSelectedSpot:=nil;
  FDrawingSpot:=TSpot.Create(skPolyLine);
end;

procedure TfrmHSEdit.FormDestroy(Sender: TObject);
begin
  FDrawingSpot.Free;
  FHotSpots.Free;
end;

procedure TfrmHSEdit.PaintBox1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
{$IFDEF FIX_LISTBOX_BUG}
 SendMessage(lBoxSpots.Handle, LB_SETCURSEL,
   FHotSpots.IndexByPoint(Point(Muldiv(X,FDenominator,FNominator),
                                                    Muldiv(Y,FDenominator,FNominator))),
                                                    0);

{$ELSE}
  lboxSpots.ItemIndex :=
     FHotSpots.IndexByPoint(Point(Muldiv(X,FDenominator,FNominator),
                                                    Muldiv(Y,FDenominator,FNominator)));


{$ENDIF}
  lboxSpotsClick(self);
end;

procedure TfrmHSEdit.lboxSpotsClick(Sender: TObject);
var
  Spot: TSpot;
  I: Integer;
begin
  UpdateButtons;
  With lboxSpots do
  begin
{$IFDEF FIX_LISTBOX_BUG}
   I := SendMessage(Handle, LB_GETCURSEL, 0, 0);
{$ELSE}
   I := ItemIndex;
{$ENDIF}

    If I < 0 then Spot := nil
      else Spot := TSpot(Items.Objects[I]);

  end;
  DrawSpot(Spot);
end;

procedure TfrmHSEdit.btnDeleteSpotClick(Sender: TObject);
var
  I: Integer;
begin
  With lboxSpots do
  begin
{$IFDEF FIX_LISTBOX_BUG}
  I := SendMessage(Handle, LB_GETCURSEL, 0, 0);
{$ELSE}
    I := ItemIndex;
{$ENDIF}
    Items.Delete(I);
  end;
  FHotSpots.DeleteSpot(I);
  lboxSpotsClick(Self);
  btnSave.Enabled := FHotSpots.Count > 0;
  btnClear.Enabled := FHotSpots.Count > 0;
end;

procedure TfrmHSEdit.btnEditSpotClick(Sender: TObject);
var
  Spot: TSpot;
  I: Integer;
begin
{$IFDEF FIX_LISTBOX_BUG}
    I := SendMessage(lboxSpots.Handle, LB_GETCURSEL, 0, 0);
{$ELSE}
    I := lboxSpots.ItemIndex;
{$ENDIF}

  Spot := FHotSpots[I];
  With TfrmSpotEdit.Create(Application) do
  try
    Caption := Format('%s[%d]',[FPrivStr, I]);
    if Edit(FGraphic, Spot)then
    begin
      Self.ScrollBox1.Update;
      DrawSpot(nil);
      With  lboxSpots do
        Items[I] := GetFirstWords(Spot.HintText);
      lboxSpotsClick(Self);
    end;
  finally
    Free;
  end;

end;

procedure TfrmHSEdit.btnNewSpotClick(Sender: TObject);
var
  Spot:TSpot;

 function GetNewSpotID:Integer;
 var
   I:integer;
 begin
   Result := 0;
   for I := 0 to FHotSpots.Count - 1 do
   with FHotSpots[I] do
     if Result < SpotID then Result := SpotID;
   inc(Result);
 end;

begin
  Spot := TSpot.Create(skPolyLine);
  Spot.SpotID := GetNewSpotID;
  With TfrmSpotEdit.Create(Application) do
  try
    Caption := Format('%s - New Spot', [FPrivStr]);
    if Edit(FGraphic, Spot)then
    begin
      Self.ScrollBox1.Update;
      FHotSpots.AddSpot(Spot);
     //DrawSpot(Spot);
      With  lboxSpots do
      begin
        Items.AddObject(GetFirstWords(Spot.HintText), Spot);
{$IFDEF FIX_LISTBOX_BUG}
        SendMessage(Handle, LB_SETCURSEL, Items.Count - 1, 0);
{$ELSE}
        ItemIndex := Items.Count - 1;
{$ENDIF}
      end;
      lboxSpotsClick(Self);
      btnSave.Enabled := true;
      btnClear.Enabled := true;
    end
     else
      Spot.Free;
  finally
    Free;
  end;
end;

procedure TfrmHSEdit.lboxSpotsDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);

var
  Spot: TSpot;

  procedure DrawKindPict;
  var
    r: Trect;
  begin
    r := Rect;
    inc(r.left, 30);
    r.right := r.left + 8;
    inc(r.top, 2);
    dec(r.Bottom, 2);

    with lboxSpots.Canvas, r do
    begin
     case Spot.SpotKind of
     skPolyLine:
      begin
        moveto(Left,Top);
        lineto(Right,Bottom);
        lineto(Left,Bottom);
        lineto(Left,Top);
      end;
     skRectangle:
       Rectangle(Left,Top,Right,Bottom);
     skEllipse:
       Ellipse(Left,Top,Right,Bottom);
     end;
    end;
  end;

  procedure DrawMaskKindPict;
  var
    r: TRect;
    mks: TMaskKinds;
  begin
    if Spot.MaskPresent then
    begin
      r := Rect;
      inc(r.Left, 45);
      r.Right := R.Left + 8;
      inc(r.Top, 5);
      dec(r.Bottom, 2);
      with lboxSpots.Canvas, r do
      begin
        mks := Spot.MaskItem.MaskKinds;
        if mkMouseOver in mks then
        begin
          MoveTo(Left, Top);
          LineTo(Right, Top);
        end;

        if mkMouseDown in mks then
        begin
          MoveTo(Left, Bottom);
          LineTo(Right, Bottom);
        end
      end;
    end;
  end;
begin

  With lboxSpots do
  begin
    Spot:=TSpot(Items.Objects[Index]);
    if odSelected in State then
    begin
      Canvas.Brush.Color := clHighlight;
      Canvas.Font.Color := clHighlightText;
      Canvas.Pen.Color := clHighlightText;
    end
     else
     begin
       if not Spot.Enabled then
       Canvas.Font.Color := clGrayText;
     end;

    Canvas.FillRect(Rect);
    With Rect do
    begin
      Canvas.TextOut(left + 1, Top, IntToStr(Spot.SpotID));
      Canvas.TextOut(60, Top, Items[Index]);
    end;
    Canvas.Pen.Color := Canvas.Font.Color;
    DrawKindPict;
    DrawMaskKindPict;
  end;
end;

procedure TfrmHSEdit.lboxSpotsDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := (Sender = Source) and (
{$IFDEF FIX_LISTBOX_BUG}
   SendMessage(lboxSpots.Handle, LB_GETCURSEL, 0, 0)
{$ELSE}
  lboxSpots.ItemIndex
{$ENDIF}
  >=0);
end;

procedure TfrmHSEdit.lboxSpotsDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  S, D: Integer;
begin
  if Sender = Source then
  begin
    With lboxSpots do
    begin
{$IFDEF FIX_LISTBOX_BUG}
      S := SendMessage(Handle, LB_GETCURSEL, 0, 0);
{$ELSE}
      S := ItemIndex;
{$ENDIF}
      D := ItemAtPos(point(X, Y), True);
      if (D >= 0) and (S <> D) then
      begin
        Items.Move(S, D);
        FHotSpots.MoveSpot(S, D);
{$IFDEF FIX_LISTBOX_BUG}
        SendMessage(Handle, LB_SETCURSEL, D, 0);
{$ELSE}
        ItemIndex := D;
{$ENDIF}
      end;
    end;
  end;
end;

function IsBinaryFormat(const FileName: string): Boolean;
begin
  Result := CompareText(ExtractFileExt(FileName), '.hmp') = 0;
end;

procedure TfrmHSEdit.btnSaveClick(Sender: TObject);
var
  fs: TFileStream;
  fn: string;
begin
  if SaveDialog1.Execute then
  begin
    fn := SaveDialog1.FileName;
    if IsBinaryFormat(fn) then
    begin
      fs := TFileStream.Create(fn,fmCreate);
      try
        FHotSpots.SaveToStream(fs);
      finally
        fs.Free;
      end;
    end
     else
      WriteHStoFile(Fn,FHotSpots);
  end;
end;

procedure TfrmHSEdit.btnLoadClick(Sender: TObject);
var
  fs: TFileStream;
  fn: string;
begin
  if OpenDialog1.Execute then
  begin
    ScrollBox1.Update;
    DrawSpot(nil);
    fn:=OpenDialog1.FileName;
    if IsBinaryFormat(fn) then
    begin
      fs:=TFileStream.Create(fn,fmOpenRead);
      try
        FHotSpots.LoadFromStream(fs);
      finally
        fs.Free;
      end;
    end
     else
       ReadHSFromFile(fn,FHotSpots);

    FillLB;
    if FHotSpots.Count > 0 then
    begin
      btnSave.Enabled := true;
      btnClear.Enabled := true;
      lboxSpots.ItemIndex :=0;
      lboxSpotsClick(self);
    end;
  end;
end;

procedure TfrmHSEdit.btnClearClick(Sender: TObject);
begin
  if MessageDlg('Do you really want to remove all Spots?',
       mtConfirmation,[mbYes,mbNo],0) <> mrYes then exit;
  DrawSpot(nil);
  FHotSpots.Clear;
  FillLB;
  btnSave.Enabled := false;
  btnClear.Enabled := false;
  lboxSpotsClick(self);
end;

procedure TfrmHSEdit.btnMaskClick(Sender: TObject);
var
  MaskItem: TMaskItem;
  R: TRect;
  I: Integer;
begin
   With lboxSpots do
{$IFDEF FIX_LISTBOX_BUG}
     I := SendMessage(Handle, LB_GETCURSEL, 0, 0);
{$ELSE}
     I := ItemIndex;
{$ENDIF}

    MaskItem := FSelectedSpot.MaskItem;
    With TfrmMaskEdit.Create(nil) do
    try
     Caption := Format('%s[%d].MaskItems', [FPrivStr, I]);
     if Edit(MaskItem, FSelectedSpot, FMaskedGraphic, FGraphic) then
     begin
      FSelectedSpot.MaskItem := MaskItem;
      With lboxSpots do
      begin
        R := ItemRect(I);
        InvalidateRect(Handle, @R, true);
      end;
     end;
    finally
      Free;
    end;
end;

end.
